# j-spring内置注解

**源码**:/j-spring/spring/SpringBoot.js

```js
const args = {
  ...,
  annotation:{
    baen:"Bean",
    valueInject:"Value",
    appBoot:"SpringBoot",
    beanInject:"Autowired",
    proxy:"Proxy",
  }
```


## 1.@Bean
> 用于指定bean的名称，配合@Autowired注解使用。  
**适用范围**：Class；**格式**：@Bean(String?)


```js
/**

  提醒：只要类名上存在任意注解标识，都可以被j-spring解析
  Example 解析后的BeanDefine {name:'example'}
  ExampleBean 解析后的 BeanDefine {name:'exampleBean'}
  AlibabaExampleBean 解析后的 BeanDefine {name:'alibaba.example.bean'}

*/

//@Bean
class Example {

}

//@Annotation
class ExampleBean {}

//@Bean(alibaba.example.bean)
class AlibabaExampleBean {}

//@Main
class Main {

  //@Autowired
  example;

  //@Autowired
  exampleBean;

  //@Autowired(alibaba.example.bean)
  ali;
}

```

## 2.@Value
> 用于向字段注入配置文件中的信息
**适用范围**：Field;**格式**:@Value(String)

配置文件：app-dev.yaml
```yaml
app:
  msg: 'hello world'
```
注入代码
```js
//@App
class App {

  /**
    强制注入yaml文件中的app.msg属性
    若无对应配置，则报错。
  */

  //@Value(app.msg)
  msg;

  /**
    尝试注入yaml文件中的app.user属性,覆盖默认值。
  */

  //@Value(value=app.user,force=false)
  user='j-spring';

}
```

## 3.@SpringBoot
> 用于标识入口Bean,只能存在一个。容器加载后，调用该Bean的main方法，并传入启动参数
**适用范围**：Class;

```js
/** 文件: index.js */
//@SpringBoot
class Application {
  main(args){
    console.log(args)//['a','1']
  }
}

//node index.js a 1
```

## 4.@Autowired
> 自动注入，只支持按BeanDefine名称导入。
**适用范围**：Field;**格式**:@Autowired(String?)

```js

//@Bean
class Bean {

  //@Autowired
  abc;

  //@Autowired(a.b.c.d)
  abcd;

  /**
    尝试注入名称为expressAppExtend的Bean,替换默认值。
    类似于接口，方便以后扩展。
  */

  //@Autowired(value=expressAppExtend,force=false)
  service = {
    extendApp(app){
    }
  };

}
```

## 5.@Proxy
> 实现动态代理，类似于java cglib;
**适用范围**：Class;**格式**:@Proxy(bean:String,method:String?)

```js

/**
 Proxy(
  bean:String, 代理指定注解的bean.
  method:<String> 可选项：代理指定方法；默认，全部方法。
)
*/

//@Proxy(bean=Service,method=Test)
class ServiceProxy {

	//方法代理
	doProxy(bean,beanDefine){
		return {
			method(wrapBean,method,args){
				return  wrapBean.invoke(convertArg)
			}
		}
	}

	//对象提升
	doEnhance(bean,beanDefine){
		//1.添加一个额外的方法
		bean.toFly = ()=>{}
		return bean;
	}

}
```

## 6.@NoProxy
> 加在被代理类的方法上，避免被代理。
**适用范围**：Method;

```js

/**
  1.该代理提升所有拥有@Application的bean。
  2.由于@Proxy没有设置method属性，所以默认代理所有方法。
*/

//@Proxy(bean=Application)
class ServiceProxy {

	//方法代理
	doProxy(bean,beanDefine){...}

	//对象提升
	doEnhance(bean,beanDefine){...}

}

 //@Application
 class Application {

   //被代理
   method(){}

   //由于拥有NoProxy注解则不会被代理。
   //@NoProxy
   method2(){}

 }

```
