# j-spring内置Bean

## 内置Bean的注入
> 注意:内置bean没有BeanDefine

```js
//@Bean
class Application {

  //@Autowired
  springFactory;

  //@Autowired
  springResource;

}
```


## SpringFactory
> 管理装配所有的BeanDefine，同样也保存了j-spring启动的上下文信息。

**常用属性和API**

```js

class SpringFactory {

  ags;// Map<String,Object> 启动参数

  classReferences;// Map<String,Class> 类的引用映射

  beanDefineList=[];// List<BeanDefine> 所有扫描解析的BeanDefine集合

  /**
    根据BeanDefine的name获取缓存中的bean。
    注意是从缓存中获取,只能获取已经实例化的bean.
  */
  @MaybeNull
  Object getBean(String beanDefineName);

  /**
    根据对象获取解析的BeanDefine
    注意也是从缓存中获取,只能获取已经装配了的BeanDefine
  */
  @MaybeNull
  BeanDefine getBeanDefineByBean(Object Bean);

  /**
    获取BeanDefine，这里是从所有解析的BeanDefine中获取。
  */
  @MaybeNull
  BeanDefine getBeanDefineByName(String beanDefineName);


  /**
    获取拥有指定注解的BeanDefine
    例如：获取SpringMvc中的Controller的BeanDefine
    const controllerBeanDefineList = getBeanDefineByAnnotation('Controller');
  */
  List<BeanDefine> getBeanDefineByAnnotation(String annotationName);

  /**
    获取拥有指定注解的bean.
    例如：获取SpringMvc中的Controller的Bean
    const controllerBeanList = getBeanByAnnotation('Controller')
  */
  List<Object> getBeanByAnnotation(String annotationName);

  /**
    根据BeanDefine创建Bean,这里会完成装配工作。
    注意：会优先返回缓存。
  */
  Object assembleBeanByBeanDefine(Beandefine beanDefine);

}
```

## SpringResource
> 管理j-spring的配置信息，支持加载多个。详见启动参数。
> 一般使用@Value(key)注入数据到字段中。详见内置注解。

app.yaml
```yaml
  app:
    user:
      name: 'bob'
      age: 15
```

**常用属性和API**
```js

class SpringResource {

  data={};//所有配置

  /**
    获取配置
    getValue('app.user') =>  {name:'bob',age:15}
    getValue('app.user.name') => 'bob'
  */
  Object getValue(String key); throw `field inject config error`

  Object getOr(key,defaultValue);

}

```
